"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * 小程序动态加载 webpack 插件
 * 更改 jsonpScript 方法, 使用 eval5 + wx.request 替代
 * @Author: Jiyu Shao
 * @Date: 2021-01-30 15:22:12
 * @Last Modified by: Jiyu Shao
 * @Last Modified time: 2021-02-04 14:10:07
 */
var webpack_1 = require("webpack");
var strip_indent_1 = __importDefault(require("strip-indent"));
var path = require('path');
var PLUGIN_NAME = 'DynamicImportWeappPlugin';
var DynamicImportWeappPlugin = /** @class */ (function () {
    function DynamicImportWeappPlugin(props) {
        var _this = this;
        /**
         * 接入 webpack, 并生成小程序动态加载代码
         * @param compiler
         */
        this.apply = function (compiler) {
            // TODO: webpack 5.0 可能会不兼容
            // 覆盖 mainTemplate.hooks.requireEnsure 和 mainTemplate.hooks.jsonpScript 来实现小程序内的动态加载
            compiler.hooks.afterPlugins.tap(PLUGIN_NAME, function () {
                compiler.hooks.compilation.tap(PLUGIN_NAME, function (compilation) {
                    var _a;
                    var mainTemplate = compilation.mainTemplate;
                    // 更改动态导入输出文件目录
                    compilation.hooks.afterOptimizeChunkIds.tap(PLUGIN_NAME, function (chunks) {
                        var dynamicOutputFolderName = path.basename(_this.options.dynamicImportFolderPath);
                        var dynamicEntryList = [];
                        chunks.forEach(function (currentChunk) {
                            if (!currentChunk.name) {
                                var currentModules = currentChunk.getModules();
                                var currentEntryModule = currentModules[currentModules.length - 1];
                                // @ts-ignore
                                var currentResource = currentEntryModule.resource;
                                if (currentResource.startsWith(_this.options.dynamicImportFolderPath)) {
                                    var currentEntryName = currentResource
                                        .substring(_this.options.dynamicImportFolderPath.length + 1)
                                        .split('.')
                                        .slice(0, -1)
                                        .join('.');
                                    dynamicEntryList.push({
                                        dynamicOutputFolderName: dynamicOutputFolderName,
                                        currentEntryName: currentEntryName,
                                    });
                                    currentChunk.name = dynamicOutputFolderName + "/" + currentEntryName;
                                    // @ts-ignore
                                    currentChunk.id = currentChunk.name;
                                    // @ts-ignore
                                    currentChunk.ids = [currentChunk.id];
                                }
                            }
                        });
                        if (dynamicEntryList.length !== 0) {
                            console.log('\n');
                            dynamicEntryList.forEach(function (currentEntry) {
                                console.log("\u7F16\u8BD1  \u53D1\u73B0\u52A8\u6001\u5165\u53E3 " + currentEntry.dynamicOutputFolderName + "/" + currentEntry.currentEntryName);
                            });
                        }
                    });
                    /**
                     *  向 Entry 中插入自定义的 loader, 来注入全局变量和引入所有的 components
                     */
                    compilation.hooks.normalModuleLoader.tap(PLUGIN_NAME, function (_, module) {
                        var isEntryModule = _this.options.isEntryModule(module);
                        if (isEntryModule) {
                            var entryLoaderPath_1 = path.resolve(__dirname, '../lib/entry-loader.js');
                            if (
                            // @ts-ignore
                            !module.loaders.some(function (item) { return item.loader === entryLoaderPath_1; })) {
                                // @ts-ignore
                                module.loaders.push({
                                    loader: entryLoaderPath_1,
                                });
                            }
                        }
                    });
                    // 注入代码 PUPLIC_PATH
                    mainTemplate.hooks.requireExtensions.tap(PLUGIN_NAME, function (source) {
                        var newSource = [];
                        newSource.push(source);
                        newSource.push('');
                        newSource.push("__webpack_require__.p = \"" + _this.options.publicPath + "\";");
                        return newSource.join('\n');
                    });
                    // 替换 jsonpScriptSrc 方法
                    mainTemplate.hooks.localVars.tap(PLUGIN_NAME, function (source) {
                        var replaceRegex = /function jsonpScriptSrc\(chunkId\) \{\n(.*)\n\}/;
                        source = source.replace(replaceRegex, strip_indent_1.default("\n              var jsonpScriptSrc = function (chunkId) {\n                return __webpack_require__.p + \"\" + chunkId + \".js\"\n              }\n            "));
                        return source;
                    });
                    // 添加小程序请求/执行代码逻辑
                    (_a = mainTemplate.hooks.jsonpScript) === null || _a === void 0 ? void 0 : _a.tap('JsonpMainTemplatePlugin', function () {
                        var chunkLoadTimeout = mainTemplate.outputOptions.chunkLoadTimeout;
                        return strip_indent_1.default("\n          var onScriptComplete;\n          // create error before stack unwound to get useful stacktrace later\n          var error = new Error();\n          onScriptComplete = function (event) {\n            clearTimeout(timeout);\n            var chunk = installedChunks[chunkId];\n            if(chunk !== 0) {\n              if(chunk) {\n                var errorType = event && (event.type === 'load' ? 'missing' : event.type);\n                var realSrc = event && event.target && event.target.src;\n                error.message = 'Loading chunk ' + chunkId + ' failed.n(' + errorType + ': ' + realSrc + ')';\n                error.name = 'ChunkLoadError';\n                error.type = errorType;\n                error.request = realSrc;\n                chunk[1](error);\n              }\n              installedChunks[chunkId] = undefined;\n            }\n          };\n          var timeout = setTimeout(function(){\n            onScriptComplete({ type: 'timeout'});\n          }, " + chunkLoadTimeout + ");\n\n          // \u4F7F\u7528\u5FAE\u4FE1\u8BF7\u6C42\u4EE3\u7801\n          var failCallback = function() {\n              onScriptComplete({\n                type: \"request:fail\",\n                target: {\n                  src: jsonpScriptSrc(chunkId)\n                }\n              })\n            }\n          var successCallback = function(res) {\n              if (res.statusCode !== 200) {\n                failCallback()\n                return;\n              }\n              try {\n                var rootContext = {\n                  wx,\n                  console,\n                  setTimeout,\n                  clearTimeout,\n                  setInterval,\n                  clearInterval,\n                }\n                // \u6267\u884C\u4EE3\u7801\n                var interpreter = new wx[\"eval5\"].Interpreter(rootContext, {\n                  rootContext,\n                });\n                interpreter.evaluate(res.data)\n              } catch (error) {\n                console.trace(error)\n              }\n            }\n\n            wx.request({\n              url: jsonpScriptSrc(chunkId),\n              timeout: " + chunkLoadTimeout + ",\n              success: successCallback,\n              fail: failCallback,\n            })");
                    });
                    // 删除 append $script 的语句
                    mainTemplate.hooks.requireEnsure.tap(PLUGIN_NAME, function (_, chunk, hash) {
                        return webpack_1.Template.asString([
                            '// JSONP chunk loading for javascript',
                            '',
                            'var installedChunkData = installedChunks[chunkId];',
                            'if(installedChunkData !== 0) { // 0 means "already installed".',
                            webpack_1.Template.indent([
                                '',
                                '// a Promise means "currently loading".',
                                'if(installedChunkData) {',
                                webpack_1.Template.indent(['promises.push(installedChunkData[2]);']),
                                '} else {',
                                webpack_1.Template.indent([
                                    '// setup Promise in chunk cache',
                                    'var promise = new Promise(function(resolve, reject) {',
                                    webpack_1.Template.indent([
                                        'installedChunkData = installedChunks[chunkId] = [resolve, reject];',
                                    ]),
                                    '});',
                                    'promises.push(installedChunkData[2] = promise);',
                                    '',
                                    '// start chunk loading',
                                    // @ts-ignore
                                    mainTemplate.hooks.jsonpScript.call('', chunk, hash),
                                ]),
                                '}',
                            ]),
                            '}',
                        ]);
                    });
                });
            });
        };
        this.options = __assign({ isEntryModule: function () { return false; } }, props);
    }
    return DynamicImportWeappPlugin;
}());
exports.default = DynamicImportWeappPlugin;
//# sourceMappingURL=index.js.map